@@ -3,7 +3,7 @@ from paver.easy import *
 from paver.setuputils import setup
 #from setuptools import setup
 from setuptools import find_packages
-import sys
+import sys, os
 sys.path.append('./src')
 import neuronvisio
 
@@ -130,7 +130,6 @@ def gh_pages_build_fix():
 @task
 def build_pdf():
     """Build the User Manual"""
-    import os
     from subprocess import call
     root = os.getcwd()
     docs = os.path.join(root, 'docs')
@@ -157,7 +156,6 @@ def build_source_deb():
     """Build a source debian package of the version supplied"""
     from subprocess import call
     import os.path
-    import neuronvisio
     package_ver = neuronvisio.__version__
     dist_dir = 'dist'
     package_name = 'neuronvisio' + '-' + package_ver + '.tar.gz'
@@ -167,10 +165,10 @@ def build_source_deb():
     call(['debuild', '-S', '-sa'])
     os.chdir('..')
 
-@task
-def upload_to_launchpad():
-    package_ver = neuronvisio.__version__
-    dist_dir = 'dist'
-    package_src_changes = 'neuronvisio_' + package_ver + '_source.changes'
-    call(['dput', 'neuronvisio-ppa', os.path.join('deb_dist', 
-                                                  package_src_changes)])
+#@task
+#def upload_to_launchpad():
+#    package_ver = neuronvisio.__version__
+#    dist_dir = 'dist'
+#    package_src_changes = 'neuronvisio_' + package_ver + '_source.changes'
+#    call(['dput', 'neuronvisio-ppa', os.path.join('deb_dist', 
+#                                                  package_src_changes)])
