@@ -45,7 +45,9 @@ class Manager(object):
         # Load the std run for NEURON
         h.load_file("stdrun.hoc")
         
-    def add_vecRef(self, var, sec, time_interval_recording=None):
+    def add_vecRef(self, var, sec, 
+                   time_interval_recording=None, 
+                   point_process=None):
         """Add the vecRef to the vec_res list. It takes care to create the vector 
         and record the given variable.
         
@@ -71,10 +73,12 @@ class Manager(object):
                             vec = h.Vector()
                             varRef = '_ref_' + var
                             if time_interval_recording is None:
-                                vec.record(getattr(sec(0.5), varRef))
+                                vec.record(point_process, 
+                                           getattr(sec(0.5), varRef))
                             else:
-                                vec.record(getattr(sec(0.5), varRef), 
-                                time_interval_recording)
+                                vec.record(point_process,
+                                           getattr(sec(0.5), varRef), 
+                                           time_interval_recording)
                             vecRef.vecs[var] = vec
                             alreadyPresent = True
                             success = True
@@ -82,33 +86,41 @@ class Manager(object):
             if not alreadyPresent:
                     
                 # Creating the vector
-                vec = h.Vector()
-                varRef = '_ref_' + var
-                try:
-                    if time_interval_recording is None:
-                        vec.record(getattr(sec(0.5), varRef))
-                    else:
-                        vec.record(getattr(sec(0.5), varRef), 
-                                   time_interval_recording)
-                except NameError:
-                    print "The variable %s is not present in the section %s" \
-                    % (varRef, sec.name())
-                    success = False
+                vec = self.create_record_vector(sec, 
+                                                var, 
+                                                time_interval_recording)
                                                 
                 # Adding to the list
                 vecRef = VecRef(sec)
                 vecRef.vecs[var] = vec
                 name = vecRef.__class__.__name__
-                t = h.Vector()
-                if time_interval_recording is None:
-                    t.record(h._ref_t)
-                else:
-                    t.record(h._ref_t, time_interval_recording)
+                if self.groups.has_key('t'):
+                    t = self.groups['t']
+                else: 
+                    t = self.create_time_record(time_interval_recording)
                 self.add_ref(vecRef, t)
-                self.groups['t'] = self.groups[name] # Adding a shortcut to the NEURON time
                 success = True
         return success
     
+    def add_all_vecRef(self, var, 
+                       time_interval_recording=None, 
+                       point_process=None):
+        """Create the vector for all the section present in the model 
+        with the given variable
+        :param var: The variable to record"""
+        
+        done = False
+        responses = []
+        for sec in h.allsec():
+            response = self.add_vecRef(var, 
+                                       sec, 
+                                       time_interval_recording,
+                                       point_process)
+            responses.append(response)
+        if all(responses) != True:
+            print "Warning: Some vectors could not be added."
+    
+    
     def add_ref(self, generic_ref, x):
         """Add a generic ref to manager.refs dictionary. If a list of the ref of 
         the same type is already present, the `genercref` will be added, otherwise the list
@@ -125,6 +137,100 @@ class Manager(object):
             self.refs[name] = [generic_ref]
             self.groups[name] = x
             
+    def add_synVecRef(self, synapse):
+        """Add the synVecRef object to the list
+        
+        :param synapse: The synapse to record.
+        """
+        synVecRef = SynVecRef(synapse.chan_type, synapse.section.name(), 
+                              synapse.vecs)
+            
+        self.add_ref(synVecRef, self.groups['t'])
+    
+    
+    
+    def find_point_process(self, sec):
+        """Find a point_process in a section if any.
+        
+        Params:
+            sec - Section to search
+            
+        Return:
+            point_process or None if not present.
+        """
+        mt = h.MechanismType(1)
+        total_mech = int(mt.count())
+        sec.push()
+        pp = None
+        for i in range(total_mech):
+            pp_type = mt.select(i)
+            pp = mt.pp_begin()
+            if pp is not None:
+                break
+            
+        h.pop_section()
+        return pp
+
+    def create_record_vector(self, sec, var, time_interval_recording):
+        """Create the vector which will record the variable
+        Will pass the first pointprocess to make sure the recording 
+        are thread safe."""
+        
+        # Getting the first pp if any
+        
+        point_process = self.find_point_process(sec)
+            
+        vec = h.Vector()
+        varRef = '_ref_' + var
+        
+        
+        try:
+            if time_interval_recording:
+                if point_process:
+                    vec.record(point_process, 
+                               getattr(sec(0.5), varRef), 
+                               time_interval_recording)
+                    print "Sec: %s has pp: %s" %(sec.name(), point_process)
+                else:
+                    vec.record(getattr(sec(0.5), varRef), 
+                               time_interval_recording)
+                    print "Sec: %s has not pp" %(sec.name())
+            else:
+                if point_process:
+                    vec.record(point_process, 
+                               getattr(sec(0.5), varRef))
+                else:
+                    vec.record(getattr(sec(0.5), varRef))
+                
+            
+        except NameError:
+            print "The variable %s is not present in the section %s" \
+            % (varRef, sec.name())
+        
+        h.pop_section()
+        return vec
+    
+    def create_time_record(self, time_interval_recording=None, point_process=None):
+        """Create the vector to record time. If not time_interval specified the 
+        NEURON default is used."""
+        t = h.Vector()
+        if time_interval_recording:
+            if point_process: 
+                t.record(point_process, 
+                         h._ref_t, time_interval_recording)
+            else:
+                t.record(h._ref_t, time_interval_recording)
+        else:
+            if point_process: 
+                t.record(point_process, 
+                         h._ref_t)
+            else:
+                t.record(h._ref_t)
+        self.groups['t'] = t
+        return t
+    
+
+            
     def get_vector(self, sec, var, group='VecRef'):
         """Return the vec that record the var in a given section
         
@@ -139,6 +245,9 @@ class Manager(object):
                 if ref.sec_name == name:
                     if ref.vecs.has_key(var):
                        return ref.vecs[var]
+    def get_time(self):
+        """Return the vector time associated with the voltage"""
+        return self.manager.groups['t']
     
     def sum_vector(self, vec1, vec2):
         """Sums two vectors with the same length. The vector are 
@@ -166,22 +275,6 @@ class Manager(object):
             vecs[sec.name()] = (vec)
         return vecs
             
-    def add_all_vecRef(self, var, time_interval_recording=None):
-        """Create the vector for all the section present in the model 
-        with the given variable
-        :param var: The variable to record"""
-        
-        done = False
-        responses = []
-        for sec in h.allsec():
-            if time_interval_recording == None:
-                response = self.add_vecRef(var, sec)
-            else: 
-                response = self.add_vecRef(var, sec, time_interval_recording)
-            responses.append(response)
-        if all(responses) != True:
-            print "Warning: Some vectors could not be added."
-            
     
     def get_tree(self, sec):
         """Return the minimal tree of section 
@@ -204,21 +297,10 @@ class Manager(object):
             parentSec = parentSeg.sec
             tree.append(parentSec)
             tree = self.__get_parent(parentSec, tree)
+        h.pop_section()
         return tree
-    
-    def add_synVecRef(self, synapse):
-        """Add the synVecRef object to the list
-        
-        :param synapse: The synapse to record.
-        """
-        synVecRef = SynVecRef(synapse.chan_type, synapse.section.name(), 
-                              synapse.vecs)
-        
-        self.add_ref(synVecRef, self.groups['t'])
-
-
-            
-    def plotVecs(self, vecs_dic, x=None, legend=True, figure_num=None, points=False):
+                
+    def plot_vecs(self, vecs_dic, x=None, legend=True, figure_num=None, points=False):
         """Plot the vectors with plt
         
         :param vecs_dic: dictionary with section name as k and the vec obj as value
@@ -395,7 +477,7 @@ class Manager(object):
                         vec = vec.to_python(tmp_array) # Swap in place
                     else:
                         vec =  vec.to_python() # Creating a list
-                print "Saving vec: %s of section: %s" %(var, section_name)
+                #print "Saving vec: %s of section: %s" %(var, section_name)
                 h5file_holder.createArray(target_group, var, 
                                           vec,
                                           title=detail)
@@ -471,6 +553,76 @@ class Manager(object):
                         genericRef.vecs = vecs
                         genericRef.detail = node._v_title
 
+    def get_distance(self, sections, base_sec):
+        """Return a dictionary with section names as keys and 
+        distance from base_sec to specified point as value in microns
+        
+        param:
+         secs: list of section which distance need to be calculated
+         base_sec: section from where the distance should be measured from."""
+         
+        distance_sec = {}
+        for sec in sections:
+            tree = self.get_tree(sec)
+            distance = self._get_distance_in_tree(tree)
+            distance -= base_sec.L
+            distance_sec[sec.name()] = distance
+        return distance_sec
+    
+    def _get_distance_in_tree(self, tree):
+        """Calculate the distance in the tree adding the length of all the sections """
+        dist = 0
+        for sec in tree:
+            dist += sec.L
+        return dist
+    
+    def create_3d_grid(self, distance_dic, time, variable_dic):
+        """Create the 3 grid (X,Y,Z) uses to plot wireframe or surface 3D 
+        
+        :param: distance_dic - dictionary with sections' names as key and 
+        distance as a value
+        :param: time - Time array
+        :param: variable_dic - dictionary with sections' names as key and 
+        array as value
+        
+        :rtype: tuple (X,Y,Z) numpy array"""
+        # create a grid
+        X_time, Y_distance = np.meshgrid(time, distance_dic.values())
+        Z_variable = X_time*0
+        xn, yn = X_time.shape 
+        sec_names = distance_dic.keys()
+        # xk iter on distance
+        for xk in range(xn):
+            # yk on time. It's the reverse of the grid.
+            for yk in range(yn):
+                var_array = variable_dic[sec_names[xk]]
+                Z_variable[xk, yk] =  var_array[yk]
+                
+        return (X_time, Y_distance, Z_variable)
+    
+    def plot3D(self, sections, base_sec, var):
+        """Wrap up module to plot in three Dimensions the voltage the distance and the time 
+        :param: sections - list of section to consider
+        :param: base_sec - section used as a starting pooint for the distance
+        :param: var - variable to plot """
+        
+        # I know it's not suggested to import here, but this is pretty new 
+        # module for matplotlib and this method is 
+        # hardely executed.
+        
+        from mpl_toolkits.mplot3d import Axes3D
+        from matplotlib import cm
+        
+        dist = self.get_distance(sections, base_sec)
+        vecs = self.get_vectors(sections, var)
+        X, Y, Z = self.create_3d_grid(dist, self.groups['t'], vecs)
+        fig = plt.figure()
+        ax = fig.gca(projection='3d')
+        surf = ax.plot_surface(X, Y, Z,  rstride=10, cstride=10, cmap=cm.jet,
+                               linewidth=0, antialiased=False)
+        fig.colorbar(surf, shrink=0.5, aspect=5)
+        plt.show()
+
 
             
 class BaseRef(object):
